from optparse import make_option
from django.core.management.base import LabelCommand, CommandError
import maintenance.api as api
import logging

logger = logging.getLogger("maintenance")


class Command(LabelCommand):
    option_list = LabelCommand.option_list + (make_option('--force', action='store_true', dest='ignore_session', default=False,
        help='Do not wiat for active session. Brutally disconnect users'),)

    args = 'on|off|check'
    label = 'command'
    help = """ """


    def handle_default_options(self):
        pass

    def handle_label(self, cmd, **options):
        if cmd not in ('on', 'off', 'check', 'activate', 'deactivate', 'status'):
            raise CommandError('Allowed options are: %s' % self.args)

        if cmd in ('check', 'status'):
            api.check()
        elif cmd in ('on', 'activate') :
            ignore_session = options.get('ignore_session')
            api.start(ignore_session)
        elif cmd in ('off', 'deactivate'):
            api.stop()









import os
from time import sleep
import time
from types import *
from django.templatetags.static import static
from django.conf import settings

def enum(**enums):
    enums['_labels'] = dict(zip(enums.values(), map(str.upper, enums.keys())))
    return type('Enum', (), enums)

STATUS = enum(STARTED=1, OFFLINE=2, ONLINE=4)
MAINTENANCE_FILE = getattr(settings, 'MAINTENANCE_FILE', '/tmp/DJANGO_MAINTENANCE_FILE_%s' % settings.SITE_ID)
PENDING_MAINTENANCE_FILE = "%s_" % MAINTENANCE_FILE
MAINTENANCE_URL = getattr(settings, 'MAINTENANCE_URL', static('maintenance/maintenance.html'))


def start(ignore_session=False):
    """ activate maintenance mode.
    this not mean that the application will be down. Check `is_active` for that.
    Logged users continue to work but no other logins will be allowed
    """
    C = ['*', '-']
    rounds = 0
    open(PENDING_MAINTENANCE_FILE, 'w').close()
    if not ignore_session:
        try:
            _start = time.time()
            while True:
                users = get_active_users()
                if not users:
                    break
                if not rounds:
                    print "Active sessions detected. Waiting for logout. (Session timeout set to %s secs) " % settings.SESSION_COOKIE_AGE
                rounds += 1
                sys.stdout.write(
                    "%s pending sessions. %s (%d sec)\r" % (users, C[rounds % 2], round(time.time() - _start)))
                sys.stdout.flush()
                sleep(1)
        except KeyboardInterrupt:
            print 'Interrupt'
            return
        finally:
            os.unlink(PENDING_MAINTENANCE_FILE)

    open(MAINTENANCE_FILE, 'w').close()
    check()


def check():
    print "Status: %s - Active sessions: %s" % ( STATUS._labels[status()], get_active_users())


def stop():
    if os.path.isfile(MAINTENANCE_FILE):
        os.unlink(MAINTENANCE_FILE)

    if os.path.isfile(PENDING_MAINTENANCE_FILE):
        os.unlink(PENDING_MAINTENANCE_FILE)
    check()


def get_active_users():
    from django.db import transaction
    from django.contrib.sessions.models import Session

    Session.objects.filter(expire_date__lt=timezone.now()).delete()
    transaction.commit_unless_managed()
    return Session.objects.count()


def is_offline():
    """ true if maintenance mode is started."""
    return os.path.exists(MAINTENANCE_FILE)


def is_pending():
    """ true if maintenance mode is pending. ie. wait for active session to expire """
    return os.path.exists(PENDING_MAINTENANCE_FILE) and not os.path.exists(MAINTENANCE_FILE)


def is_online():
    """ true if maintenance mode is off """
    return not is_offline()


def status():
    if is_offline():
        return STATUS.OFFLINE
    elif is_pending():
        return STATUS.PENDING
    elif is_online():
        return STATUS.ONLINE





from django.conf import settings
from django.http import HttpResponseRedirect
from django.utils.importlib import import_module
from maintenance import api



class MaintenanceMiddleware(object):
    """ this middleware intercept the request and redirect ``MAINTENANCE_URL`` with this policy:

    * State: is_pending()
        - Logged user are allowed to work. :ref:`maintenance.context_processors.maintenance` should be used to inform them for the pending offline mode
        - New user's requests will bre redirectd to ``MAINTENANCE_URL``

    * State: is_offline()
        - Nobody will be able to access to the site, but it's only a double chyou should  not be here
    """

    def process_request(self, request):
        if api.MAINTENANCE_URL == request.path:
            return None

        if api.is_offline():
            return HttpResponseRedirect(api.MAINTENANCE_URL)

        if  api.is_pending():
            engine = import_module(settings.SESSION_ENGINE)
            session_key = request.COOKIES.get(settings.SESSION_COOKIE_NAME, None)
            session = engine.SessionStore()
            if session.exists(session_key):
                return

        return None



import logging
from django.conf import settings
from django.utils.importlib import import_module
from maintenance.management.lockfile import FileLock, LockFailed, AlreadyLocked, LockTimeout

__all__ = ['register_session', 'unregister_session', 'CommandTask', 'CommandRunningError']
logger = logging.getLogger("commands")

class CommandRunningError(Exception):
    pass


def register_session(session_key):
    engine = import_module(settings.SESSION_ENGINE)
    session = engine.SessionStore(session_key)
    if session.exists(session_key):
        raise CommandRunningError()
    session.create()

def unregister_session(session_key):
    engine = import_module(settings.SESSION_ENGINE)
    session = engine.SessionStore(session_key)
    session.delete()

class CommandTask( object ):
    def __init__( self, session_key, force=False, timeout=1 ):
        self._key= session_key
        self.lock = FileLock(self._key)

        if force:
            self.lock.break_lock()

        try:
            self.lock.acquire(timeout)
        except (AlreadyLocked, LockFailed, LockTimeout), e:
            raise CommandRunningError('Unable to lock', e)

    def __enter__( self ):
        register_session(self._key)

    def __exit__( self, type, value, tb ):
        self.lock.release()
        unregister_session(self._key)
        if type:
            logger.exception(value)
            raise type(value)
